USE [Prince]
GO
/****** Object:  UserDefinedFunction [Common].[UDF_Gregorian_To_Persian_NEW]    Script Date: 03/21/2025 16:33:04 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
ALTER FUNCTION [Common].[UDF_Gregorian_To_Persian]
(
    @GregorianDate DATE
)
RETURNS NVARCHAR(10)
AS
BEGIN
    DECLARE @JDN BIGINT;
    DECLARE @Depoch BIGINT;
    DECLARE @Cycle BIGINT;
    DECLARE @Cyear BIGINT;
    DECLARE @Ycycle BIGINT;
    DECLARE @Aux1 BIGINT;
    DECLARE @Aux2 BIGINT;
    DECLARE @Yday BIGINT;
    DECLARE @PersianYear INT;
    DECLARE @PersianMonth INT;
    DECLARE @PersianDay INT;

    -- Calculate Julian Day Number (JDN) for the Gregorian date
    SET @JDN = DATEDIFF(DAY, '0001-01-01', @GregorianDate) + 1721425;

    -- Calculate Persian year, month, and day
    SET @Depoch = @JDN - [COMMON].[UDF_Persian_To_Julian](475, 1, 1);
    SET @Cycle = FLOOR(@Depoch / 1029983);
    SET @Cyear = @Depoch % 1029983;

    IF @Cyear = 1029982
    BEGIN
        SET @Ycycle = 2820;
    END
    ELSE
    BEGIN
        SET @Aux1 = FLOOR(@Cyear / 366);
        SET @Aux2 = @Cyear % 366;
        SET @Ycycle = FLOOR((2134 * @Aux1 + 2816 * @Aux2 + 2815) / 1028522) + @Aux1 + 1;
    END

    SET @PersianYear = @Ycycle + 2820 * @Cycle + 474;

    IF @PersianYear <= 0
    BEGIN
        SET @PersianYear = @PersianYear - 1;
    END

    SET @Yday = @JDN - [COMMON].[UDF_Persian_To_Julian](@PersianYear, 1, 1) + 1;

    IF @Yday <= 186
    BEGIN
        SET @PersianMonth = CEILING(@Yday / 31.0);
        SET @PersianDay = @Yday - (@PersianMonth - 1) * 31;
    END
    ELSE
    BEGIN
        SET @PersianMonth = CEILING((@Yday - 186) / 30.0) + 6;
        SET @PersianDay = (@Yday - 186) - (@PersianMonth - 7) * 30;
    END

    -- Format Persian date as YYYY/MM/DD
    RETURN 
        CAST(@PersianYear AS NVARCHAR(4)) + '/' + 
        RIGHT('0' + CAST(@PersianMonth AS NVARCHAR(2)), 2) + '/' + 
        RIGHT('0' + CAST(@PersianDay AS NVARCHAR(2)), 2);
END;